package cn.timer.api.controller.insure;

import cn.timer.api.bean.insure.*;
import cn.timer.api.bean.yggl.YgglMainEmp;
import cn.timer.api.utils.HttpUtils;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.google.common.collect.Maps;
import com.mysql.cj.util.StringUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.codec.digest.DigestUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.io.InputStream;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Random;

/**
 * @Description TODO
 * @Author wgd
 * @Date 2022/3/3 8:45
 */
@Api(tags = "8.0回调接口")
@RestController
@Transactional
@RequestMapping(value = "/callBack/policy", produces = {"application/json"})
public class CallBackContorll {
    private static final Logger log = LoggerFactory.getLogger(CallBackContorll.class);
    @Value("${insure.appid}")
    private String appid;
    @Value("${insure.secret}")
    private String secret;
    /*保全测试用*/
    @Value("${insure.appidq}")
    private String appidq;
    @Value("${insure.secretq}")
    private String secretq;

    @Value("${insure.getPolicyUrl}")
    private String getPolicyUrl;

    @Value("${BASE_API_URL}")
    private String base_api_url;


    private Map setParams(String sign, String appid, String secret) {
        /*当前时间戳*/
        long timestamp = System.currentTimeMillis() / 1000;
        log.info("时间戳" + timestamp);
        Map paramsMap = Maps.newHashMap();
        paramsMap.put("pid", appid);
        paramsMap.put("timestamp", String.valueOf(timestamp));
        paramsMap.put("trace_id", appid + timestamp + new Random().nextInt((9999 - 100) + 1) + 10);
//        paramsMap.put("sign",Md5.md5(appid+secret+timestamp+sign.trim()));
        String value;
        if (sign == null) {
            value = appid + secret + timestamp;
        } else {
            value = appid + secret + timestamp + sign;
            log.info("body参数======" + sign);
        }
        paramsMap.put("sign", DigestUtils.md5Hex(value));
        log.info("params参数======" + JSONObject.toJSONString(paramsMap));
        return paramsMap;
    }


    @PostMapping(value = "/insuredCallBack")
    @ApiOperation(value = "6.投保申请回调", httpMethod = "POST", notes = "投保申请回调")
    private Map insuredCallBack(HttpServletRequest request, @RequestParam String pid, @RequestParam String sign, @RequestParam String timestamp) throws IOException {
        Map map = Maps.newHashMap();
        map.put("status", "error");
        if (StringUtils.isNullOrEmpty(pid) || StringUtils.isNullOrEmpty(sign) && StringUtils.isNullOrEmpty(timestamp)) {
            return map;
        }
        if (!pid.equals(appidq)) {
            return map;
        }
        InputStream is = null;
        is = request.getInputStream();
        StringBuilder sb = new StringBuilder();
        byte[] b = new byte[4096];
        for (int n; (n = is.read(b)) != -1; ) {
            sb.append(new String(b, 0, n));
        }
        String value = DigestUtils.md5Hex(appidq + secretq + timestamp + sb.toString());
        if (!value.equals(sign)) {
            return map;
        }
        CallBack callBack = JSONObject.parseObject(sb.toString(), CallBack.class);
        Map trueMap = Maps.newHashMap();
        trueMap.put("status", "1");
        return trueMap;
    }

    @PostMapping(value = "/CallBack")
    @ApiOperation(value = "7.保全增员申请回调", httpMethod = "POST", notes = "投保申请回调")
    private Map callBack(HttpServletRequest request, @RequestParam String pid, @RequestParam String sign, @RequestParam String timestamp) throws IOException {
        Map map = Maps.newHashMap();
        map.put("status", "error");
        if (StringUtils.isNullOrEmpty(pid) || StringUtils.isNullOrEmpty(sign) && StringUtils.isNullOrEmpty(timestamp)) {
            return map;
        }
        if (!pid.equals(appidq)) {
            return map;
        }
        InputStream is = null;
        is = request.getInputStream();
        StringBuilder sb = new StringBuilder();
        byte[] b = new byte[4096];
        for (int n; (n = is.read(b)) != -1; ) {
            sb.append(new String(b, 0, n));
        }
        String value = DigestUtils.md5Hex(appidq + secretq + timestamp + sb.toString());
        if (!value.equals(sign)) {
            return map;
        }
        Map paramsMap = Maps.newHashMap();
        paramsMap.put("pid", pid);
        paramsMap.put("timestamp", timestamp);
        paramsMap.put("sign", sign);
        CallBack callBack = JSONObject.parseObject(sb.toString(), CallBack.class);
        /*1保全复核回调*/
        if (callBack.getCallback_type().equals("1")) {
            /*核保*/
        } else {

            /*到这就已经支付成功出单了*/
            log.info("callBack=====" + sb.toString());
            if (callBack.getStatus().equals("1")) {
                List<InsureUser> list = InsureUser.builder().build().selectList(new QueryWrapper<InsureUser>().lambda().eq(InsureUser::getBatchNo, callBack.getOrder_import_info().getUuid()).eq(InsureUser::getInsureStatus, 3));
                list.forEach(i -> {
                    i.setInsureStatus(1);
                    i.updateById();
                    YgglMainEmp.builder().isInsure(1).build().update(new QueryWrapper<YgglMainEmp>().lambda().eq(YgglMainEmp::getId, i.getUserId()));
                });
                List<InsureUser> oldlist = InsureUser.builder().build().selectList(new QueryWrapper<InsureUser>().lambda().eq(InsureUser::getReplaceTransId, callBack.getOrder_import_info().getUuid()));
                if (oldlist != null && oldlist.size() > 0) {
                    oldlist.forEach(i -> {
                        i.setInsureStatus(4);
                        i.setStatus("2");
                        i.updateById();
                        YgglMainEmp.builder().isInsure(0).build().update(new QueryWrapper<YgglMainEmp>().lambda().eq(YgglMainEmp::getId, i.getUserId()));
                    });
                }
                InsurePolicy insurePolicy = InsurePolicy.builder().build().selectOne(new QueryWrapper<InsurePolicy>().lambda().eq(InsurePolicy::getPolicyNo, list.get(0).getPolicyNo()));
                if (oldlist == null || oldlist.size() == 0) {
                    insurePolicy.setTotalPremium(String.valueOf(Double.valueOf(insurePolicy.getTotalPremium()) + Double.valueOf(callBack.getOrder_import_info().getTotal_money())));
                }
                insurePolicy.setPolicyFile(callBack.getOrder_import_info().getEndorsement_file());
                insurePolicy.setUpdateTime(new Date());
                insurePolicy.updateById();
                InsureLog.builder().requestParam(JSONObject.toJSONString(paramsMap)).type(7)
                        .requestData(sb.toString()).createTime(new Date()).requestType(1).returnBody(JSONObject.toJSONString(callBack)).requestPath(base_api_url + "/callBack/policy/CallBack")
                        .returnCode(callBack.getStatus()).returnMsg("更新成功").policyId(insurePolicy.getId()).build().insert();
            } else {
                String errorMsg = "";
                InsureLog insureLog = InsureLog.builder().build().selectOne(new QueryWrapper<InsureLog>().lambda().eq(InsureLog::getTransId, callBack.getOrder_import_info().getThird_uuid()));
                List<Map> errMap = callBack.getOrder_import_info().getErr_list();
                if (errMap.size() > 0) {
                    for (int i = 0; i < errMap.size(); i++) {
                        errorMsg = errorMsg + ("姓名：" + errMap.get(i).get("name").toString() + "，错误：" + errMap.get(i).get("err_content").toString() + ',');
                    }
                } else {
                    errorMsg = callBack.getErr_msg();
                }
                //TODO 写入日志
                InsureLog.builder().requestParam(JSONObject.toJSONString(paramsMap)).type(7)
                        .requestData(sb.toString()).createTime(new Date()).requestType(1).returnBody(JSONObject.toJSONString(callBack)).requestPath(base_api_url + "/callBack/policy/CallBack")
                        .returnCode(callBack.getStatus()).returnMsg(errorMsg).policyId(insureLog.getPolicyId()).build().insert();
            }

        }
        Map trueMap = Maps.newHashMap();
        trueMap.put("status", "1");

        return trueMap;
    }

    @GetMapping(value = "/payStatus")
    @ApiOperation(value = "8.支付完成跳转", httpMethod = "GET", notes = "用于支付时跳回我们系统更新状态")
    private void callBackPayStatus(HttpServletRequest request, @RequestParam Integer policyId) throws IOException {
        log.info("支付完成回调（仅代表用户点了已支付，无法确认是否真正支付）");
        InsureLog.builder().type(7).createTime(new Date()).requestType(2).requestPath(base_api_url + "/payStatus?policyId=" + policyId)
                .returnCode("suc").returnMsg("用户已支付，等待更新保单状态").policyId(policyId).build().insert();
    }

    @PostMapping(value = "/payCallBack")
    @ApiOperation(value = "9.投保支付收银台回调", httpMethod = "POST", notes = "支付完成跳转")
    private Map payCallBack(HttpServletRequest request, @RequestParam String pid, @RequestParam String sign, @RequestParam String timestamp) throws IOException {
        log.info("支付回调");
        Map map = Maps.newHashMap();
        InputStream is = null;
        is = request.getInputStream();
        StringBuilder sb = new StringBuilder();
        byte[] b = new byte[4096];
        for (int n; (n = is.read(b)) != -1; ) {
            sb.append(new String(b, 0, n));
        }
        String value = DigestUtils.md5Hex(appid + secret + timestamp + sb.toString());
        if (!value.equals(sign)) {
            return map;
        }
        Map paramsMap = Maps.newHashMap();
        paramsMap.put("pid", pid);
        paramsMap.put("timestamp", timestamp);
        paramsMap.put("sign", sign);
        PayCallBack callBack = JSONObject.parseObject(sb.toString(), PayCallBack.class);
        InsurePay insurePay = InsurePay.builder().build().selectOne(new QueryWrapper<InsurePay>().lambda().eq(InsurePay::getSerialNumber, callBack.getSerial_number()));
        insurePay.setAmount(Double.valueOf(callBack.getAmount()));
        insurePay.setPayStatus(Integer.parseInt(callBack.getPay_status()));
        insurePay.setPaySerialNo(callBack.getPay_serial_no());
        insurePay.setSerialNumber(callBack.getSerial_number());
        insurePay.setPayType(callBack.getPay_type());
        insurePay.updateById();
        /*调用出单接口更新保单状态*/
        Map bodyMap = Maps.newHashMap();
        bodyMap.put("quotation_id", callBack.getSerial_number());
        String data = HttpUtils.sendPost(getPolicyUrl, setParams(JSONObject.toJSONString(bodyMap), appid, secret), bodyMap);
        Map dataMap = JSONObject.parseObject(data, Map.class);
        if (dataMap.size() > 0) {
            if (dataMap.get("errcode").toString().equals("e25")) {
                //TODO 如果是E25则将保单设为出单中，更新交由保单出单回调做,设为出单状态是因为之后可以手动校验
                InsurePolicy insurePolicy = InsurePolicy.builder().id(insurePay.getPolicyId()).build().selectById();
                insurePolicy.setStatus("2");
                insurePolicy.setUpdateTime(new Date());
                insurePolicy.updateById();
                InsureLog.builder().type(7).createTime(new Date()).requestType(1).requestPath(base_api_url + "/payCallBack")
                        .returnCode("suc").returnMsg(dataMap.get("errmsg").toString()).build().insert();
            } else if (dataMap.get("errcode").toString().equals("suc")) {
                InsureLog.builder().type(7).createTime(new Date()).requestType(1).requestPath(base_api_url + "/payCallBack")
                        .returnCode("suc").returnMsg("确认支付成功，支付方式：" + insurePay.getPayType() + "，支付金额：" + callBack.getAmount()).policyId(insurePay.getPolicyId()).build().insert();
            } else {
                InsureLog.builder().requestParam(JSONObject.toJSONString(setParams(JSONObject.toJSONString(bodyMap), appid, secret))).type(7)
                        .requestData(JSONObject.toJSONString(bodyMap)).createTime(new Date()).requestType(1).returnBody(data).requestPath(getPolicyUrl)
                        .returnCode(dataMap.get("errcode").toString()).policyId(insurePay.getPolicyId()).returnMsg(dataMap.get("errmsg").toString()).build().insert();
            }
        }
        map.put("status", "1");
        return map;
    }

    @PostMapping(value = "/issueCallback")
    @ApiOperation(value = "10.保单出单回调", httpMethod = "POST", notes = "支付完成跳转")
    private Map issueCallback(HttpServletRequest request, @RequestParam String pid, @RequestParam String sign, @RequestParam String timestamp) throws IOException {
        log.info("保单出单回调");
        Map map = Maps.newHashMap();
        map.put("status", "1");
        InputStream is = null;
        is = request.getInputStream();
        StringBuilder sb = new StringBuilder();
        byte[] b = new byte[4096];
        for (int n; (n = is.read(b)) != -1; ) {
            sb.append(new String(b, 0, n));
        }
        String value = DigestUtils.md5Hex(appid + secret + timestamp + sb.toString());
        if (!value.equals(sign)) {
            return map;
        }
        PolicyCallBack callBack = JSONObject.parseObject(sb.toString(), PolicyCallBack.class);
        InsurePay insurePay = InsurePay.builder().build().selectOne(new QueryWrapper<InsurePay>().lambda().eq(InsurePay::getSerialNumber, callBack.getSerial_number()));
        InsurePolicy insurePolicy = InsurePolicy.builder().id(insurePay.getPolicyId()).build().selectById();
        List<InsureUser> userList = InsureUser.builder().build().selectList(new QueryWrapper<InsureUser>().lambda().eq(InsureUser::getPolicyId, insurePolicy.getId()));
        insurePolicy.setUpdateTime(new Date());
        if (callBack.getStatus().equals("1")) {
            insurePay.setPayStatus(2);
            insurePolicy.setPolicyNo(callBack.getPolicy_no());
            insurePolicy.setTotalPremium(callBack.getTotal_premium());
            insurePolicy.setPolicyFile(callBack.getPolicy_file());
            insurePolicy.setStatus("1");
            insurePolicy.setKitUrl(callBack.getKit_url());
            userList.forEach(u -> {
                u.setStatus("1");
                u.setInsureStatus(1);
                u.updateById();
            });
        } else {
            insurePay.setPayStatus(3);
            insurePolicy.setStatus("3");
            userList.forEach(u -> {
                u.setStatus("2");
                u.setInsureStatus(2);
                u.updateById();
            });
            insurePolicy.updateById();
            insurePay.updateById();
        }
        InsureLog.builder().type(7).createTime(new Date()).requestType(1).returnBody(sb.toString()).requestPath(getPolicyUrl)
                .returnCode(callBack.getStatus()).policyId(insurePay.getPolicyId()).returnMsg(callBack.getErr_msg()).build().insert();
        return map;
    }
}
